//
// Copyright (c) 2000 Daniel Edelson. All rights reserved.
//

package org.myworldgis.io.shapefile;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import java.io.IOException;
import java.io.RandomAccessFile;
import javax.measure.converter.UnitConverter;


/**
 *
 */
public final strictfp class ESRIShapefileReader implements ESRIShapeConstants {
    
    //--------------------------------------------------------------------------
    // Instance variables
    //--------------------------------------------------------------------------
    
    /** */
    private RandomAccessFile _file;
    
    /** */
    private ESRIShapeBuffer _buffer;
    
    /** */
    private Envelope _envelope;
    
    /** */
    private int _shapeType;
    
    /** */
    private int _fileSizeBytes;
    
    //--------------------------------------------------------------------------
    // Constructors
    //--------------------------------------------------------------------------
    
    /** */
    public ESRIShapefileReader (RandomAccessFile file, 
                                UnitConverter inputConverter,
                                GeometryFactory factory) throws IOException {
        _file = file;
        _buffer = new ESRIShapeBuffer(DEFAULT_BUFFER_SIZE, inputConverter, factory);
        _file.seek(0);
        _buffer.read(_file, 0, SHAPE_FILE_HEADER_LENGTH);
        _envelope = _buffer.getBoundingBox(36);
        _shapeType = _buffer.getInt(32);
        _buffer.setByteOrder(ESRIShapeBuffer.ByteOrder.BIG_ENDIAN);
        _fileSizeBytes = _buffer.getInt(24) * 2;
        _buffer.setByteOrder(ESRIShapeBuffer.ByteOrder.LITTLE_ENDIAN);
    }
    
    //--------------------------------------------------------------------------
    // instance methods
    //--------------------------------------------------------------------------
    
    /** */
    public Envelope getEnvelope () {
        return _envelope;
    }
    
    /** */
    public int getShapeType () {
        return _shapeType;
    }
    
    /** */
    public int getSizeBytes () {
        return _fileSizeBytes;
    }
    
    /** */
    public Geometry getShape (ESRIShapeIndexRecord index) throws IOException {
        _file.seek(index.getOffsetBytes());
        _buffer.ensureCapacity(SHAPE_RECORD_HEADER_LENGTH + index.getSizeBytes());
        _buffer.read(_file, 0, SHAPE_RECORD_HEADER_LENGTH + index.getSizeBytes());
        return _buffer.getESRIRecord(0);
    }
    
    /** */
    public Geometry getNextShape () throws IOException {
        int bytesRead = _buffer.read(_file, 0, SHAPE_RECORD_HEADER_LENGTH);
        if (bytesRead < SHAPE_RECORD_HEADER_LENGTH) {
            return null;
        }
        _buffer.setByteOrder(ESRIShapeBuffer.ByteOrder.BIG_ENDIAN);
        int contentByteLength = _buffer.getInt(4) * 2;
        _buffer.ensureCapacity(SHAPE_RECORD_HEADER_LENGTH + contentByteLength);
        _buffer.setByteOrder(ESRIShapeBuffer.ByteOrder.LITTLE_ENDIAN);
        _buffer.read(_file, SHAPE_RECORD_HEADER_LENGTH, contentByteLength);
        return _buffer.getESRIRecord(0);
    }
    
    /** */
    public void close () throws IOException {
        _file.close();
    }    
}
