//
// Copyright (c)2006 by Northwestern University. All Rights Reserved.
// 

package org.myworldgis.util;

import com.vividsolutions.jts.geom.Envelope;
import java.awt.Dimension;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.util.Locale;

/** */
public class WMSUtils {
    
    
    //--------------------------------------------------------------------------
    // Inner classes
    //--------------------------------------------------------------------------

    public enum ImageFormat {
        
        JPEG("image/jpeg"),
        PNG("image/png"),
        GIF("image/gif");
        
        private final String _mimeType;
        
        ImageFormat (String mimeType) {
            _mimeType = mimeType;
        }
        
        public String mimeType () {
            return _mimeType;
        }
    }
    
    //--------------------------------------------------------------------------
    // Class variables
    //--------------------------------------------------------------------------
    
    /** */
    private static final NumberFormat NUMBER_FORMAT = new DecimalFormat("#########0.#####", new DecimalFormatSymbols(Locale.US));
    
    //--------------------------------------------------------------------------
    // Class methods
    //--------------------------------------------------------------------------
    
    /** */
    public static String makeGetMapURL (String serverURL,
                                        Envelope viewBounds,
                                        Dimension pixelDimensions,
                                        String srs,
                                        String layers,
                                        ImageFormat imageFormat) {
        StringBuilder buffer = new StringBuilder();
        buffer.append(serverURL);
        if (!serverURL.endsWith("?")) {
            buffer.append("?");
        }
        buffer.append("&");
        buffer.append("Service=wms");
        buffer.append("&");
        buffer.append("Version=1.1.1");
        buffer.append("&");
        buffer.append("Request=GetMap");
        buffer.append("&");
        buffer.append("SRS=");
        buffer.append(srs);
        buffer.append("&");
        buffer.append("Layers=");
        buffer.append(layers);
        buffer.append("&");
        buffer.append("BBOX=");
        buffer.append(NUMBER_FORMAT.format(viewBounds.getMinX()));
        buffer.append(",");
        buffer.append(NUMBER_FORMAT.format(viewBounds.getMinY()));
        buffer.append(",");
        buffer.append(NUMBER_FORMAT.format(viewBounds.getMaxX()));
        buffer.append(",");
        buffer.append(NUMBER_FORMAT.format(viewBounds.getMaxY()));
        buffer.append("&");
        // TerraServer restricts sizes to between 50-2000, 
        // so we'll just enforce that for everyone whose size 
        // isn't fixed
        int width = Math.min(Math.max(pixelDimensions.width, 50), 2000);
        int height = Math.min(Math.max(pixelDimensions.height, 50), 2000);
        buffer.append("Width=");
        buffer.append(width);
        buffer.append("&");
        buffer.append("Height=");
        buffer.append(height);
        buffer.append("&");
        buffer.append("Format=");
        buffer.append(imageFormat.mimeType());
        buffer.append("&");
        buffer.append("Styles=");
        return(buffer.toString());
    }
    
    /** */
    public static String makeGetCapabilitiesURL (String server) {
        StringBuilder buffer = new StringBuilder();
        buffer.append(server);
        buffer.append("?");
        buffer.append("Version=1.1.1");
        buffer.append("&");
        buffer.append("Request=GetCapabilities");
        buffer.append("&");
        buffer.append("Service=wms");
        return(buffer.toString());
    }
}
